function [height,width] = plotLowerLevelOfGSIPImplicitly1D(obj,hsp, indexOfLowLev,x,lb,ub,varargin)

             
set(hsp,'Units','centimeter');
oldPos=get(hsp,'Position');

width=oldPos(3);
height=min(width/3.5,oldPos(4));
set(hsp,'Position',[oldPos(1),oldPos(2),width,height]);
set(hsp,'XGrid','on');
hold(hsp,'on')
set(get(gca,'parent'), 'currentaxes', hsp);

%SetNumberOfPoints and initialize points
NrPoints=1000;
X=linspace(lb,ub,NrPoints)';


%Find semi-infinite constraints that belong to the given indexsets
indicesOfSemiInfConstrs=zeros(1,0);
for i=1:obj.getNrSemiInfConstrs
     index=obj.getIndexSetOfSemiInfConstr(i);
     if any(index==indexOfLowLev)
         indicesOfSemiInfConstrs=[indicesOfSemiInfConstrs,i];
     end
end


%%plot the semi-infinite constraints
%evaluate points:
g=maxOfSemiInfConstrs(X);

gPosNeg=g>0;
Index=1;
crit=true; 
while crit
    %determine the severel Intervals
    
    %leftBound first non negative value
    [minleft,leftIndex]=min(gPosNeg);
    gPosNeg=gPosNeg(leftIndex+1:end);
    leftIndex=Index+leftIndex;
    Index=leftIndex;
    if minleft==1
        break
    end

    %rigtht Bound last non negative value
    [maxright,rightIndex]=max(gPosNeg);
    gPosNeg=gPosNeg(rightIndex:end);
    rightIndex=rightIndex+Index-1;
    Index=rightIndex;

    %plot the detected Interval
    if maxright==0
        rightIndex=NrPoints;
    else
        %plot right bracket !!Attention special cases
        bracketIndex=max(leftIndex,rightIndex-round(1/20*NrPoints));
        plot(hsp,[X(bracketIndex),X(rightIndex),X(rightIndex),X(bracketIndex)],[0.25,0.25,0.75,0.75],'-b');
    end
    
    h=fill([X(leftIndex),X(leftIndex),X(rightIndex),X(rightIndex)],[0.25,0.75,0.75,0.25],[135/255, 206/255, 250/255]);
    set(h, 'EdgeColor', 'none','facealpha',0.5);
    %plot left bracket !!Attention special cases
    if leftIndex~=1
        bracketIndex=min(leftIndex+round(1/20*NrPoints),rightIndex);
        plot(hsp,[X(bracketIndex),X(leftIndex),X(leftIndex),X(bracketIndex)],[0.25,0.25,0.75,0.75],'-b');
    end
    %check criterion
    crit=rightIndex~=NrPoints;
end



%%plot the lower level constraints
%evaluate points:
u=maxOfIndexSet(X);

uPosNeg=u>0;
Index=1;
crit=true; 
while crit
    %determine the severel Intervals
    
    %leftBound first non negative value
    [minleft,leftIndex]=min(uPosNeg);
    uPosNeg=uPosNeg(leftIndex+1:end);
    leftIndex=Index+leftIndex;
    Index=leftIndex;
    if minleft==1
        break
    end

    %rigtht Bound last non negative value
    [maxright,rightIndex]=max(uPosNeg);
    uPosNeg=uPosNeg(rightIndex:end);
    rightIndex=rightIndex+Index-1;
    Index=rightIndex;

    %plot the detected Interval
    if maxright==0
        rightIndex=NrPoints;
    else
        %plot right bracket !!Attention special cases
        bracketIndex=max(leftIndex,rightIndex-(1/25*NrPoints));
        plot(hsp,[X(bracketIndex),X(rightIndex),X(rightIndex),X(bracketIndex)],[0.35,0.35,0.65,0.65],'-','color',[0,0.5,0]);
    end
    h=fill([X(leftIndex),X(leftIndex),X(rightIndex),X(rightIndex)],[0.35,0.65,0.65,0.35],[0,1,0]);
    set(h, 'EdgeColor', 'none','facealpha',0.5);
    %plot left bracket !!Attention special cases
    if leftIndex~=1
        bracketIndex=min(leftIndex+round(1/25*NrPoints),rightIndex);
        plot(hsp,[X(bracketIndex),X(leftIndex),X(leftIndex),X(bracketIndex)],[0.35,0.35,0.65,0.65],'-','color',[0,0.5,0]);
    end
    %check criterion
    crit=rightIndex~=NrPoints;
end




%%plot the points given in yred and yblue
nrPoints=(nargin-6)/2;
for k=1:nrPoints
    Y=varargin{2*k-1};
    color=varargin{2*k};

    if size(Y,1)~=0
        scatter(hsp,Y,0.5*ones(size(Y)),'MarkerEdgeColor','black','MarkerFaceColor',color)
    end
end

%%plot the line for the Real line
plot(hsp,[lb,ub],[0.5,0.5],'-','color','black')
set(hsp,'YTickLabel',[]);
ylabel(hsp,'');
xlabel(hsp,'y')
xlim(hsp,[lb(1), ub(1)]);
ylim(hsp,[0, 1]);
title(hsp,sprintf('Containment condition of the indexset %i',indexOfLowLev));

%functions that collect all constraints
    function g=maxOfSemiInfConstrs(y1)
        try
        g=-inf;
        for i=indicesOfSemiInfConstrs
            g=max(g,obj.evalSemiInfConstrs(i,x,y1));
        end
        catch me
            warning('warning failed to evaluate on array inputs.')
            g=-inf*ones(NrPoints,1);
            for j=1:NrPoints
                for i=indicesOfSemiInfConstrs
                    g(j)=max(g,obj.evalSemiInfConstrs(i,x,y1(j)));
                end
            end
        end
        
        
    end
    function u=maxOfIndexSet(y1)
        u=-inf;
        %collect non linear constraints
        if obj.getNrNonlinLowLevConstrs(indexOfLowLev)>0
            u_temp=obj.evalLowLevNonlinConstrs(indexOfLowLev,x,y1);
            assert(size(u_temp,1)==size(y1,1));
            for i=1:obj.getNrNonlinLowLevConstrs(indexOfLowLev)            
                u=max(u,u_temp(:,i));          
            end
        end
            
        %collect bounds
        [problb,probub]=obj.getBoundsOnLowLevVars(indexOfLowLev,x);
        u=max(u,problb(1)-y1); 
        u=max(u,y1-probub(1));
        
        %collect linear constraints
        [A,b]=obj.getLinIneqsOnLowLevVars(indexOfLowLev,x);
        for i=1:size(A,1)
            u=max(u,y1*A(i,:)'-b(i));
        end
            
        
    end




end


