classdef Problem10<ImplicitGSIPProblem
    % p=Problem10 creates an instance of the GSIP problem:
    %
    %       min   (x1-0.25)^2+x2^2
    %
    %       s.t.  y+x2 <=0 for all y in Y(x)
    %             10^-6 <=x1 <= 1
    %             -1 <=x2 <= 1
    %
    %       where:
    %
    %       Y(x)={y in R with
    %              y^2-x1 <= 0
    %              -1 <= y <= 1}
    

    properties(GetAccess=protected)

        
        nrOfUppLevVars=2;                     
        boundsOnUppLevVars=[-1,-1;1,1];        
        nrOfSemiInfConstrs=1;                 
        initValsOfUppLevVars=[0.5,0.5];     
        
        linEqsOnUppLevVars=[];                   
        linIneqsOnUppLevVars=[-1,0,-1.e-6];      
        nrOfNonlinEqsUppLev=0;                
        nrOfNonlinIneqsUppLev=0;              
        
        nrOfLowLevVars=1;                         
        nrOfLowLevNonlinConstrs=1;                       
        
        con2LowLev=1;
        nrOfIndexset=1;
        
    end
    
    methods
        %objective function
        function f=evalObjFun(~,x)                    
            f=(x(1)-1/4)^2+x(2)^2;
        end
        function grad_f=gradObjFun(~,x)                   
            grad_f=[2*(x(1)-1/4);2*x(2)];
        end
        
        %upper level
        function g=evalSemiInfConstrs(~,~,x,y)          
                                                        
            g=y+x(2);            
        end
        function Dx_g=DxSemiInfConstrs(~,~,~,y)      
            m=size(y,1);
            Dx_g=[zeros(m,1),ones(m,1)];                
        end
        function Dy_g=DySemiInfConstrs(~,~,~,y)
            m=size(y,1);
            Dy_g=ones(m,1);                
        end
                     
        %lower level
        function u=evalLowLevNonlinConstrs(~,~,x,y)           
            u=[y^2-x(1)] ;                        
        end
        function Dy_u=DyLowLevNonlinConstrs(~,~,~,y)              
            Dy_u=[2*y];                        
        end
        function Dx_u=DxLowLevNonlinConstrs(~,~,x,~)
            Dx_u=[-1,0];
        end
        
        function y=getInitValsLowLevVars(~,~,x)           
            y=(-1+x(1))/2;                
        end
        function Dx_y=DxStartingPointLowLev(~,~,~)
            Dx_y=[0.5,0];
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(obj,~,~)
             lb=-1;
             ub=1;
        end
        
        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            [lb,ub]=obj.getInterval(x);
            y=lb*z+ub*(1-z);
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            [lb,ub]=obj.getInterval(x);
            z=(y-ub)/((lb-ub));
                        
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
            [Dx_lb,Dx_ub]=obj.DxInterval(x);
            Dx_y=zeros(1,size(x,2),size(z,1));
            o=ones(size(z));
          
            Dx_y(1,:,:)=(o*Dx_lb*z+(1-z)*(Dx_ub))';
        end
    end
    methods(Access=private)
        
        
        function [lb,ub]=getInterval(~,x)                
            lb=-sqrt(abs(x(1)));
            ub=sqrt(abs(x(1)));            
        end
        function [Dx_lb,Dx_ub]=DxInterval(~,x)
            Dx_lb=[-sign(x(1))/2/sqrt(abs(x(1))),0];
            Dx_ub=[sign(x(1))/2/sqrt(abs(x(1))),0];            
        end
        
       
    end
end

    

    