classdef Problem4< ImplicitGSIPProblem
    % p=Problem4() creates an instance of the GSIP problem:
    %
    %       min   x4
    %
    %       s.t.  3(x1-y)^2 + (2-y)x2^2+5(x3^2+y)+2x1+3x2-x3+e^(4y^2)- x4 for all y in Y(x) 
    %
    %       where:
    %
    %       Y(x)={y in R with
    %               y   <= -0.25sin(x1x2)+0.5
    %               0 <= y <= 1}
    
    properties(GetAccess=protected)

        
        nrOfUppLevVars=4;                         
        boundsOnUppLevVars=[-inf,-inf,-inf,-inf;inf,inf,inf,inf];
        nrOfSemiInfConstrs=1;                       
        initValsOfUppLevVars=[0,0,0,0];            
        
        linEqsOnUppLevVars=[];                     
        linIneqsOnUppLevVars=[];                   
        nrOfNonlinEqsUppLev=0;                
        nrOfNonlinIneqsUppLev=0;                      
        
        nrOfLowLevVars=1;                          
        nrOfLowLevNonlinConstrs=0;                        
        
        con2LowLev=1;
        nrOfIndexset=1;
        
    end
    
    methods
        %objective function
        function f=evalObjFun(~,x)                    
            f=x(4);
        end
        function grad_f=gradObjFun(~,~)                    
            grad_f=[0,0,0,1]';
        end
        
        %upper level
        function g=evalSemiInfConstrs(~,~,x,y)          
                                                           
            g=3*(x(1)-y).^2+(2-y)*x(2)^2+5*(x(3)^2+y)+2*x(1)+3*x(2)-x(3)+exp(4*y.^2)-x(4);
        end
        function Dx_g=DxSemiInfConstrs(~,~,x,y)      
            m=size(y,1);
            o=ones(m,1);
            Dx_g=[6*(x(1)-y)+2,2*(2-y)*x(2)+3,10*x(3)*o-1,-1*o];                
        end
        function Dy_g=DySemiInfConstrs(~,~,x,y)
            Dy_g=-6*(x(1)-y)-x(2)^2+5+8*y.*exp(4*y^2);                
        end
               
        
        
        %lower level
            
        function y=getInitValsLowLevVars(~,~,~)         
            y=1/8;                
        end
        function Dx_y=DxStartingPointLowLev(~,~,~)
            Dx_y=zeros(1,4);
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(obj,~,~)  
             lb=0;
             ub=1;
        end
                
        %Linear inequalities
        function [A,b]=getLinIneqsOnLowLevVars(obj,k,x)
            A=[1];
            b=[-1/4*sin(x(1)*x(2))+0.5];
        end
        
        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            [lb,ub]=obj.getInterval(x);
            y=lb*z+ub*(1-z);
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            [lb,ub]=obj.getInterval(x);
            z=(y-ub)/((lb-ub));
                        
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
            [Dx_lb,Dx_ub]=obj.DxInterval(x);
            Dx_y=zeros(1,size(x,2),size(z,1));
            o=ones(size(z));
          
            Dx_y(1,:,:)=(o*Dx_lb*z+(1-z)*(Dx_ub))';
        end
    end
    methods(Access=private)
        
        function [lb,ub] = getInterval(~,x)           
            lb=0;
            ub=1/2-1/4*sin(x(1)*x(2));
        end
        function [Dx_lb,Dx_ub]=DxInterval(~,x)
            Dx_lb=zeros(1,4);
            Dx_ub=[-1/4*x(2)*cos(x(1)*x(2)),-1/4*x(1)*cos(x(1)*x(2)),0,0];
        end

    end
    
end
