
classdef Problem8< ImplicitGSIPProblem
    % p=Problem8(m) creates an instance of the GSIP problem:
    %
    %       min   sum_i=1^m(3(m-i+1)/m*xi^2-2xi)
    %
    %       s.t.  y+sum_i=1^m(xi^2) -7 <=0 for all y in Y(x) 
    %
    %       where:
    %
    %       Y(x)={y in R with
    %              -100 <= y <= sum_i=1^m(3i/m*xi^2)-6}
 
    
    properties(GetAccess=protected)
        
        nrOfUppLevVars;                         
        boundsOnUppLevVars;                      
        nrOfSemiInfConstrs=1;                    
        initValsOfUppLevVars;                   
        
        linEqsOnUppLevVars=[];                  
        linIneqsOnUppLevVars=[];                
        nrOfNonlinEqsUppLev=0;             
        nrOfNonlinIneqsUppLev=0;           
        
        
        nrOfLowLevVars=1;                       
        nrOfLowLevNonlinConstrs=0;
        
        con2LowLev=1;
        nrOfIndexset=1;        

        
    end
    
    
    methods
        %constructor
        function p=Problem8(nrOfParameters)        
            p.nrOfUppLevVars=nrOfParameters;
            p.initValsOfUppLevVars=zeros(1,nrOfParameters);
            p.boundsOnUppLevVars=[-inf*ones(1,nrOfParameters);inf*ones(1,nrOfParameters)];
        end
    end
    
    methods 
        %objective function
        function f=evalObjFun(obj,x)               
            m=obj.nrOfUppLevVars;
            elementsSum=3*(m-(1:m)+1)/m.*(x.^2)-2*x;
            f=sum(elementsSum);
        end
        function grad_f=gradObjFun(obj,x)                   
            n=obj.nrOfUppLevVars;
            grad_f=(6*(n-(1:n)+1)/n.*x-2)';
        end
        
        function g=evalSemiInfConstrs(~,~,x,y)      
            g=y+sum(x.^2)-7;            
        end
        function Dx_g=DxSemiInfConstrs(~,~,x,y)
            m=size(y,1);
            
            Dx_g=repmat(2*x,m,1);                
        end
        function Dy_g=DySemiInfConstrs(~,~,~,y)
            m=size(y,1);
            Dy_g=ones(m,1);                
        end
             
        %lower level
        
        function y=getInitValsLowLevVars(~,~,~)          
            y=0;                
        end
        function Dx_y=DxStartingPointLowLev(obj,~,~)
            Dx_y=zeros(1,obj.nrOfUppLevVars);
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(obj,k,x)   
            m=obj.nrOfUppLevVars;
            lb=-100;
            ub=sum(3*(1:m)/m.*x.^2)-6;
        end
                        
        
        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            [lb,ub]=obj.getInterval(x);
            y=lb*z+ub*(1-z);
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            [lb,ub]=obj.getInterval(x);
            z=(y-ub)/((lb-ub));
                        
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
            [Dx_lb,Dx_ub]=obj.DxInterval(x);
            Dx_y=zeros(1,size(x,2),size(z,1));
            o=ones(size(z));
          
            Dx_y(1,:,:)=(o*Dx_lb*z+(1-z)*(Dx_ub))';
        end
    end
    methods(Access=private)
        
        function [lb,ub]=getInterval(obj,x)               
            n=obj.nrOfUppLevVars;
            lb=-100;
            ub=sum(3*(1:n)/n.*x.^2)-6;
        end
        function [Dx_lb,Dx_ub]=DxInterval(obj,x)
            n=obj.nrOfUppLevVars;
            Dx_lb=zeros(1,n);
            Dx_ub=6*(1:n)/n.*x;
        end

       
    end

    

    
end

